/*
 *
 * tests/test-RGBAL.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/utils/AppUtils.H>
#include <nucleo/image/Image.H>
#include <nucleo/image/sink/ImageSink.H>
#include <nucleo/core/ReactiveEngine.H>

#include <stdexcept>
#include <iostream>
#include <cstdlib>

#include <libgen.h>

using namespace nucleo ;

int
main(int argc, char **argv) {
  try {
    int WIDTH = 600 ;
    int HEIGHT = 400 ;
    char *ENCODING = "RGB" ;
    char *OUTPUT = "glwindow:" ;
    int firstArg = parseCommandLine(argc, argv, "w:h:e:o:", "iiss",
							 &WIDTH, &HEIGHT, &ENCODING, &OUTPUT) ;
    if (firstArg<0) {
	 std::cerr << "Usage: " << basename(argv[0]) << " [-w width] [-h height] [-e encoding] [-o output]" << std::endl ;
	 exit(-1) ;
    }

    Image image ;
    Image::Encoding encoding = Image::getEncodingByName(ENCODING) ;
    image.prepareFor(WIDTH, HEIGHT, encoding) ;

    unsigned char *ptr = image.getData() ;
    unsigned int bpp = image.getBytesPerPixel() ;
    for (unsigned int i=0; i<bpp; ++i) {
	 int x1 = i*WIDTH/bpp ;
	 int x2 = x1 + WIDTH/bpp ;
	 for (int y=0; y<HEIGHT; ++y)
	   for (int x=x1; x<x2; ++x) {
		unsigned char *pixel = ptr + (y*WIDTH+x)*bpp ;
		for (unsigned int c=0; c<bpp; ++c)
		  *pixel++ = c==i ? 255 : 0 ;
	   }
    }
    
    ImageSink *sink = ImageSink::create(OUTPUT) ;
    sink->start() ;
    sink->handle(&image) ;

    ReactiveEngine::run() ;

  } catch (std::runtime_error e) {
    std::cerr << "Runtime error: " << e.what() << std::endl ;
  } catch (std::exception e) {
    std::cerr << "Error: " << e.what() << std::endl ;
  }


  return 0 ;
}
